@@ -28,7 +28,6 @@ from hamcrest import assert_that, equal_to, is_not
 
 from behave4cli import command_shell, command_util, pathutil, textutil
 from behave4cli.command_shell_proc import BehaveWinCommandOutputProcessor, TextProcessor
-from behave4cli.pathutil import posixpath_normpath
 
 # NOT-USED: from hamcrest import contains_string
 
@@ -147,15 +146,16 @@ def step_use_directory_as_working_directory(context, directory):
 def step_a_file_named_filename_and_encoding_with(context, filename, encoding):
     """Creates a textual file with the content provided as docstring."""
     __encoding_is_valid = True
+
     text_to_use = context.text
-    # TODO refactor this try/except
     try:
-        if context.surrogate_text is not None:
+        if context.surrogate_text:
             text_to_use = context.surrogate_text
-        else:
-            assert context.text is not None, "ENSURE: multiline text is provided."
+            context.surrogate_text = None
     except AttributeError:
-        assert context.text is not None, "ENSURE: multiline text is provided."
+        pass
+
+    assert text_to_use is not None, "ENSURE: multiline text is provided."
     assert not os.path.isabs(filename)
     assert __encoding_is_valid
     command_util.ensure_workdir_exists(context)
@@ -261,10 +261,12 @@ def step_it_should_pass_with(context):
             """
     '''
     assert context.text is not None, "ENSURE: multiline text is provided."
-    step_command_output_should_contain(context)
     assert_that(
-        context.command_result.returncode, equal_to(0), context.command_result.output
+        context.command_result.returncode,
+        equal_to(0),
+        "Expected to pass but failed with content:\n" + context.command_result.output,
     )
+    step_command_output_should_contain(context)
 
 
 @then("it should fail with:")
@@ -280,8 +282,12 @@ def step_it_should_fail_with(context):
             """
     '''
     assert context.text is not None, "ENSURE: multiline text is provided."
+    assert_that(
+        context.command_result.returncode,
+        is_not(equal_to(0)),
+        "Expected to fail but passed with content:\n" + context.command_result.output,
+    )
     step_command_output_should_contain(context)
-    assert_that(context.command_result.returncode, is_not(equal_to(0)))
 
 
 # -----------------------------------------------------------------------------
@@ -294,13 +300,7 @@ def step_command_output_should_contain_text(context, text):
         ...
         Then the command output should contain "TEXT"
     """
-    expected_text = text
-    if "{__WORKDIR__}" in expected_text or "{__CWD__}" in expected_text:
-        expected_text = textutil.template_substitute(
-            text,
-            __WORKDIR__=posixpath_normpath(context.workdir),
-            __CWD__=posixpath_normpath(os.getcwd()),
-        )
+    expected_text = command_util.template_substitute_builtin_expressions(context, text)
     actual_output = context.command_result.output
     with on_assert_failed_print_details(actual_output, expected_text):
         textutil.assert_normtext_should_contain(actual_output, expected_text)
@@ -313,13 +313,7 @@ def step_command_output_should_not_contain_text(context, text):
         ...
         then the command output should not contain "TEXT"
     """
-    expected_text = text
-    if "{__WORKDIR__}" in text or "{__CWD__}" in text:
-        expected_text = textutil.template_substitute(
-            text,
-            __WORKDIR__=posixpath_normpath(context.workdir),
-            __CWD__=posixpath_normpath(os.getcwd()),
-        )
+    expected_text = command_util.template_substitute_builtin_expressions(context, text)
     actual_output = context.command_result.output
     with on_assert_failed_print_details(actual_output, expected_text):
         textutil.assert_normtext_should_not_contain(actual_output, expected_text)
@@ -333,13 +327,7 @@ def step_command_output_should_contain_text_multiple_times(context, text, count)
         Then the command output should contain "TEXT" 3 times
     """
     assert count >= 0
-    expected_text = text
-    if "{__WORKDIR__}" in expected_text or "{__CWD__}" in expected_text:
-        expected_text = textutil.template_substitute(
-            text,
-            __WORKDIR__=posixpath_normpath(context.workdir),
-            __CWD__=posixpath_normpath(os.getcwd()),
-        )
+    expected_text = command_util.template_substitute_builtin_expressions(context, text)
     actual_output = context.command_result.output
     with on_assert_failed_print_details(actual_output, expected_text):
         textutil.assert_normtext_should_contain_multiple_times(
@@ -358,26 +346,14 @@ def step_command_output_should_contain_exactly_text(context, text):
         When I run "echo Hello"
         Then the command output should contain "Hello"
     """
-    expected_text = text
-    if "{__WORKDIR__}" in text or "{__CWD__}" in text:
-        expected_text = textutil.template_substitute(
-            text,
-            __WORKDIR__=posixpath_normpath(context.workdir),
-            __CWD__=posixpath_normpath(os.getcwd()),
-        )
+    expected_text = command_util.template_substitute_builtin_expressions(context, text)
     actual_output = context.command_result.output
     textutil.assert_text_should_contain_exactly(actual_output, expected_text)
 
 
 @then('the command output should not contain exactly "{text}"')
 def step_command_output_should_not_contain_exactly_text(context, text):
-    expected_text = text
-    if "{__WORKDIR__}" in text or "{__CWD__}" in text:
-        expected_text = textutil.template_substitute(
-            text,
-            __WORKDIR__=posixpath_normpath(context.workdir),
-            __CWD__=posixpath_normpath(os.getcwd()),
-        )
+    expected_text = command_util.template_substitute_builtin_expressions(context, text)
     actual_output = context.command_result.output
     textutil.assert_text_should_not_contain_exactly(actual_output, expected_text)
 
@@ -585,13 +561,7 @@ def step_remove_file(context, filename):
 # -----------------------------------------------------------------------------
 @then('the file "{filename}" should contain "{text}"')
 def step_file_should_contain_text(context, filename, text):
-    expected_text = text
-    if "{__WORKDIR__}" in text or "{__CWD__}" in text:
-        expected_text = textutil.template_substitute(
-            text,
-            __WORKDIR__=posixpath_normpath(context.workdir),
-            __CWD__=posixpath_normpath(os.getcwd()),
-        )
+    expected_text = command_util.template_substitute_builtin_expressions(context, text)
     file_contents = pathutil.read_file_contents(filename, context=context)
     file_contents = file_contents.rstrip()
     if file_contents_normalizer:
